'use strict';
//插件
var router = require('express').Router(),
    returnCode = require('../../utils/returnCode'),
    _ = require('lodash'),
    then = require('thenjs'),
    moment = require('moment'),
    mongoose = require('mongoose'),
    util = require('../../utils/util'),
    user = require('../../utils/user'),
    redisPraiseLog = require('../../utils/redisPraiseLog'),
    async = require('async');
//模型    
var ForumThread = mongoose.model('ForumThread'),
    ForumComment = mongoose.model('ForumComment'),
    ForumPVLog = mongoose.model('ForumPVLog'),
    ForumFollowThread = mongoose.model('ForumFollowThread');
//服务    
var forumThreadService = require('../../service/forumThreadService'),
    forumUserService = require('../../service/forumUserService'),
    forumCommentService = require('../../service/forumCommentService'),
    forumAboutMEService = require('../../service/forumAboutMEService');

module.exports = function(app) {
    app.use('/admin/forum', router);
};

//格式化日期 (格式：年-月-日)
function date_format(date) {
    return moment(date).format('YYYY-MM-DD HH:mm:ss');
}

//文章列表
router.get('/threadManagement/threads/list', function(req, res, next) {
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;
    var infoId = req.query.infoId;
    var tagId = req.query.tagId;
    var pid = req.query.pid;//父文章id
    var content = req.query.content;//标题 或 内容
    var type = req.query.type;//1普通文章,话题,照片墙
    var nickName = req.query.nickName;
    var mid = req.query.mid;
    var begin_time=req.query.begin_time;
    var end_time=req.query.end_time;
    var tab = req.query.tab;//全部,官方贴,推荐贴,置顶帖,加精贴,活动贴,屏蔽贴
    var asc = req.query.asc === 'true'?1:-1;
    var sortBy = {//默认排序
        created:asc
    };
    if(req.query.sort){//指定排序字段
        sortBy = {};
        sortBy[req.query.sort] = asc;
        sortBy['_id'] = -1;
    }
    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1,
        status: {
            $ne: 3
        }
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }
    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }
    if (content) {
        conditions.$or = [
            {
                title:{
                    $regex: content,
                    $options: 'i'
                }
            },
            {
                content:{
                    $regex: content,
                    $options: 'i'
                }
            }
        ];
    }
    if (type) {
        conditions.type = Number(type);
    }
    if(tab){
        if(tab === 'admin'){//官方贴
            conditions.isPublishByBg = 1;
        }
        else if(tab === 'new_recommend'){//推荐贴
            conditions.new_recommend = 1;
        }
        else if(tab === 'top'){//置顶贴
            conditions.top = 1;
        }
        else if(tab === 'recommend'){//加精贴
            conditions.recommend = 1;
        }
        else if(tab === 'event'){//活动贴
            conditions.isEvent = 1;
        }
        else if(tab === 'closed'){//屏蔽贴
            conditions.status = 0;
        }
    }

    if(begin_time && end_time){
        conditions.created = {$gte : begin_time, $lte : end_time};
    } else if(end_time){
        conditions.created = {$lte : end_time};
    } else if(begin_time){
        conditions.created = {$gte : begin_time};
    }

    var callback = function(err, results) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            if (pid) {
                forumThreadService.getById(pid, function(err, parentThread) {
                    res.json(_.assign({
                        parentThread: parentThread
                    }, results, returnCode.SUCCESS));
                });
            } else {
                res.json(_.assign(results, returnCode.SUCCESS));
            }
        }
    };
    if (mid) {
        forumThreadService.getAllThreadByFidAndMid(mid, conditions, pageNo, pageSize, sortBy, callback);
    } else if (nickName) {
        forumThreadService.getAllThreadByFidAndNickName(nickName, conditions, pageNo, pageSize, sortBy, callback);
    } else {
        forumThreadService.getAllThreadByFid(conditions, pageNo, pageSize, sortBy, callback);
    }
});

//删除查询结果的所有帖子
router.post('/threadManagement/threads/list/deleteAll', function(req, res, next) {
    var infoId = req.body.infoId;
    var tagId = req.body.tagId;
    var pid = req.body.pid;//父文章id
    var content = req.body.content;//标题 或 内容
    var type = req.body.type;//1普通文章,话题,照片墙
    var nickName = req.body.nickName;
    var mid = req.body.mid;
    var begin_time=req.body.begin_time;
    var end_time=req.body.end_time;
    var tab = req.body.tab;//全部,官方贴,推荐贴,置顶帖,加精贴,活动贴,屏蔽贴
    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1,
        status: {
            $ne: 3
        }
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }
    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }
    if (content) {
        conditions.$or = [
            {
                title:{
                    $regex: content,
                    $options: 'i'
                }
            },
            {
                content:{
                    $regex: content,
                    $options: 'i'
                }
            }
        ];
    }
    if (type) {
        conditions.type = Number(type);
    }
    if(tab){
        if(tab === 'admin'){//官方贴
            conditions.isPublishByBg = 1;
        }
        else if(tab === 'new_recommend'){//推荐贴
            conditions.new_recommend = 1;
        }
        else if(tab === 'top'){//置顶贴
            conditions.top = 1;
        }
        else if(tab === 'recommend'){//加精贴
            conditions.recommend = 1;
        }
        else if(tab === 'event'){//活动贴
            conditions.isEvent = 1;
        }
        else if(tab === 'closed'){//屏蔽贴
            conditions.status = 0;
        }
    }

    if(begin_time && end_time){
        conditions.created = {$gte : begin_time, $lte : end_time};
    } else if(end_time){
        conditions.created = {$lte : end_time};
    } else if(begin_time){
        conditions.created = {$gte : begin_time};
    }

    var callback = function(err, result) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(_.assign({
                data:result
            }, returnCode.SUCCESS));
        }
    };
    var deleteAll= function(condition,callback){
        ForumThread.find(conditions).select('_id').exec(function(err, docs) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                var _tasks=[];
                _.forEach(docs,function(doc){
                    _tasks.push(function(cb){
                        forumThreadService.logicDeleteThreadById(doc._id, function(err) {
                            if (err) {
                                cb(err, null);
                            } else {
                                forumAboutMEService.updateThreadStatus(doc._id, 3);
                                cb(null, true);
                            }
                        });
                    });
                });
                var chunks=_.chunk(_tasks, 100);
                var tasks = [];
                _.forEach(chunks,function(chunk){
                    tasks.push(function(cb){
                        async.parallel(chunk,function(err){
                            cb(err, true);
                        });
                    });
                });
                async.series(tasks,function(err){
                    callback(err, true);
                });
            }
        });
    };
    if (mid) {
        forumUserService.searchMembersByMid(mid, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户的文章
                var user_ids = [];
                for(var i in users){
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    "$in" : user_ids
                };
                deleteAll(conditions,callback);
            }
        });
    } else if (nickName) {
        //查询到用户
        forumUserService.searchMembersByNickName(nickName, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户的文章
                var user_ids = [];
                for(var i in users){
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    "$in" : user_ids
                };
                deleteAll(conditions,callback);
            }
        });
    } else {
        deleteAll(conditions,callback);
    }
});

//屏蔽查询结果的所有帖子
router.post('/threadManagement/threads/list/closeAll', function(req, res, next) {
    var infoId = req.body.infoId;
    var tagId = req.body.tagId;
    var pid = req.body.pid;//父文章id
    var content = req.body.content;//标题 或 内容
    var type = req.body.type;//1普通文章,话题,照片墙
    var nickName = req.body.nickName;
    var mid = req.body.mid;
    var begin_time=req.body.begin_time;
    var end_time=req.body.end_time;
    var tab = req.body.tab;//全部,官方贴,推荐贴,置顶帖,加精贴,活动贴,屏蔽贴
    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1,
        status: {
            $ne: 3
        }
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }
    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }
    if (content) {
        conditions.$or = [
            {
                title:{
                    $regex: content,
                    $options: 'i'
                }
            },
            {
                content:{
                    $regex: content,
                    $options: 'i'
                }
            }
        ];
    }
    if (type) {
        conditions.type = Number(type);
    }
    if(tab){
        if(tab === 'admin'){//官方贴
            conditions.isPublishByBg = 1;
        }
        else if(tab === 'new_recommend'){//推荐贴
            conditions.new_recommend = 1;
        }
        else if(tab === 'top'){//置顶贴
            conditions.top = 1;
        }
        else if(tab === 'recommend'){//加精贴
            conditions.recommend = 1;
        }
        else if(tab === 'event'){//活动贴
            conditions.isEvent = 1;
        }
        else if(tab === 'closed'){//屏蔽贴
            conditions.status = 0;
        }
    }

    if(begin_time && end_time){
        conditions.created = {$gte : begin_time, $lte : end_time};
    } else if(end_time){
        conditions.created = {$lte : end_time};
    } else if(begin_time){
        conditions.created = {$gte : begin_time};
    }

    var callback = function(err, result) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(_.assign({
                data:result
            }, returnCode.SUCCESS));
        }
    };
    var closeAll= function(condition,callback){
        ForumThread.find(conditions).select('_id').exec(function(err, docs) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                var _tasks=[];
                _.forEach(docs,function(doc){
                    _tasks.push(function(cb){
                        forumThreadService.updateThreadById(doc._id, {status:0}, function(err) {
                            if (err) {
                                cb(err, null);
                            } else {
                                forumAboutMEService.updateThreadStatus(doc._id, 0);
                                cb(null, true);
                            }
                        });
                    });
                });
                var chunks=_.chunk(_tasks, 100);
                var tasks = [];
                _.forEach(chunks,function(chunk){
                    tasks.push(function(cb){
                        async.parallel(chunk,function(err){
                            cb(err, true);
                        });
                    });
                });
                async.series(tasks,function(err){
                    callback(err, true);
                });
            }
        });
    };
    if (mid) {
        forumUserService.searchMembersByMid(mid, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户的文章
                var user_ids = [];
                for(var i in users){
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    "$in" : user_ids
                };
                closeAll(conditions,callback);
            }
        });
    } else if (nickName) {
        //查询到用户
        forumUserService.searchMembersByNickName(nickName, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户的文章
                var user_ids = [];
                for(var i in users){
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    "$in" : user_ids
                };
                closeAll(conditions,callback);
            }
        });
    } else {
        closeAll(conditions,callback);
    }
});

//获取帖子总数,帖子数,用户发帖数,帖子加精数
router.get('/threadManagement/threads/statistics', function(req, res, next) {
    var infoId = req.query.infoId;
    var tagId = req.query.tagId;
    var pid = req.query.pid;//父文章id
    var content = req.query.content;//标题 或 内容
    var type = req.query.type;//1普通文章,话题,照片墙
    var nickName = req.query.nickName;
    var mid = req.query.mid;
    var begin_time=req.query.begin_time;
    var end_time=req.query.end_time;
    var tab = req.query.tab;//全部,官方贴,推荐贴,置顶帖,加精贴,活动贴,屏蔽贴

    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1,
        status: {
            $ne: 3
        }
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }
    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }
    if (content) {
        conditions.$or = [
            {
                title:{
                    $regex: content,
                    $options: 'i'
                }
            },
            {
                content:{
                    $regex: content,
                    $options: 'i'
                }
            }
        ];
    }
    if (type) {
        conditions.type = Number(type);
    }
    if(tab){
        if(tab === 'admin'){//官方贴
            conditions.isPublishByBg = 1;
        }
        else if(tab === 'new_recommend'){//推荐贴
            conditions.new_recommend = 1;
        }
        else if(tab === 'top'){//置顶贴
            conditions.top = 1;
        }
        else if(tab === 'recommend'){//加精贴
            conditions.recommend = 1;
        }
        else if(tab === 'event'){//活动贴
            conditions.isEvent = 1;
        }
        else if(tab === 'closed'){//屏蔽贴
            conditions.status = 0;
        }
    }

    if(begin_time && end_time){
        conditions.created = {$gte : begin_time, $lte : end_time};
    } else if(end_time){
        conditions.created = {$lte : end_time};
    } else if(begin_time){
        conditions.created = {$gte : begin_time};
    }

    async.parallel([function(cb){//获取帖子总数
        var _conditions = {
            ent_code: req.session.user.ent_code,
            level: 1,
            status: {
                $ne: 3
            }
        };
        ForumThread.count(_conditions, function(err, count) {
            cb(err,['thread_total',count || 0]);
        });
    },function(cb){//获取帖子数
        var _conditions = _.assign({}, conditions);
        if (mid) {//查询对应用户的文章
            forumUserService.searchMembersByMid(mid, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_total',count || 0]);
                    });
                }

            });
        } else if (nickName) {
            forumUserService.searchMembersByNickName(nickName, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_total',count || 0]);
                    });
                }
            });
        } else {
            ForumThread.count(_conditions, function(err, count) {
                cb(err,['thread_search_total',count || 0]);
            });
        }
    },function(cb){//获取用户发帖数
        var _conditions = _.assign({isPublishByBg:{$ne:1}}, conditions);
        if (mid) {//查询对应用户的文章
            forumUserService.searchMembersByMid(mid, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_user_publish',count || 0]);
                    });
                }

            });
        } else if (nickName) {
            forumUserService.searchMembersByNickName(nickName, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_user_publish',count || 0]);
                    });
                }
            });
        } else {
            ForumThread.count(_conditions, function(err, count) {
                cb(err,['thread_search_user_publish',count || 0]);
            });
        }
    },function(cb){//获取加精帖子数
        var _conditions = _.assign({recommend:1}, conditions);
        if (mid) {//查询对应用户的文章
            forumUserService.searchMembersByMid(mid, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_recommend',count || 0]);
                    });
                }

            });
        } else if (nickName) {
            forumUserService.searchMembersByNickName(nickName, function(err, users) {
                if (err) {
                    cb(err, null);
                } else {
                    var user_ids = [];
                    if(users){
                        _.forEach(users,function(user){
                            user_ids.push(user._id);
                        });
                    }
                    _conditions.from = {
                        "$in" : user_ids
                    };
                    ForumThread.count(_conditions, function(err, count) {
                        cb(err,['thread_search_recommend',count || 0]);
                    });
                }
            });
        } else {
            ForumThread.count(_conditions, function(err, count) {
                cb(err,['thread_search_recommend',count || 0]);
            });
        }
    }],function(err,results){
        if(err){
            console.log(err);
            return res.json(returnCode.BUSY);
        }
        // 数组转对象_.zipObject([[‘fred‘, 30], [‘barney‘, 40]]);→ { ‘fred‘: 30, ‘barney‘: 40 }
        res.json(_.assign({
            data:_.zipObject(results)
        }, returnCode.SUCCESS));
    });

});

//批量屏蔽帖子
router.post('/threadManagement/threads/batchclose', function(req, res, next) {
    var ent_code = req.session.user.ent_code;
    var ids = req.body.ids;
    if(!ent_code || !ids){
        return res.json(returnCode.BUSY);
    }
    forumThreadService.batchClose(ent_code,ids,function(err,result){
        if(err){
            return res.json(returnCode.BUSY);
        }
        return res.json(_.assign({data:true}, returnCode.SUCCESS));
    });
});

//批量删除帖子
router.post('/threadManagement/threads/batchdel', function(req, res, next) {
    var ent_code = req.session.user.ent_code;
    var ids = req.body.ids;
    if(!ent_code || !ids){
        return res.json(returnCode.BUSY);
    }
    forumThreadService.batchLogicDelete(ent_code,ids,function(err,result){
        if(err){
            return res.json(returnCode.BUSY);
        }
        return res.json(_.assign({data:true}, returnCode.SUCCESS));
    });
});

function updateComemntStatus(ent_code,id,status,cb){
    if(!ent_code || !id){
        return cb && cb(null,true);
    }
    var cid = id;
    async.waterfall([
        function(callback) {
            //获取评论实例
            forumCommentService.getCommentById(cid, callback);
        },
        function(comment, callback) {
            //更新评论状态
            forumCommentService.updateCommentStatusById(cid, status, function(err, update) {
                callback(err, comment);
            });
        },
        function(comment, callback) {
            if (comment && comment.level == 1) {
                forumAboutMEService.updateCommentLevel1Status(comment._id, status);
            }
            if (comment && comment.level == 2) {
                forumAboutMEService.updateCommentLevel2Status(comment._id, status);
            }
            //是否删除评论
            if (status == 2) {
                if (comment.level == 1) {//一级评论,需要移除文章的comments里的cid，并且评论数-1
                    forumThreadService.updateThreadById(comment.thread, {
                        $pull:{
                            comments:cid
                        },
                        $inc: {
                            comment_count: -1
                        }
                    }, function(err, result) {
                        if(err){
                            console.log(err);
                        }
                        callback();
                    });
                } else {//二级评论，移除父级评论的comments里的cid
                    forumCommentService.getCommentParent(cid, function(err, p_comment) {
                        if(err || !p_comment){
                            console.log(err);
                        }else{
                            forumCommentService.updateCommentById(p_comment._id, {
                                $pull:{
                                    comments:cid
                                },
                                $inc: {
                                    comment_count: -1
                                }
                            }, function(err, update) {
                                if(err){
                                    console.log(err);
                                }
                            });
                        }
                        callback();
                    });
                }
            } else {
                callback();
            }
        }
    ], function(err, restult) {
        if (err) {
            console.error(err);
        }
        cb(err,!!!err);
    });
}
function batchUpdateCommentStatus(ent_code,ids,status,callback){
    if(!ent_code || !ids){
        return callback && callback('参数错误');
    }
    var tasks = [];
    _.forEach(ids,function(id){
        tasks.push(function(cb){
            updateComemntStatus(ent_code,id,status,function(err,result){
                cb();
            })
        });
    });
    async.parallel(tasks,function(err,results){
        return callback && callback(err,!!!err);
    });
}

//批量屏蔽帖子的评论
router.post('/threadManagement/threads/:id/comment/batchclose', function(req, res, next) {
    var ent_code = req.session.user.ent_code;
    var id = req.params.id;//帖子id
    var ids = req.body.ids;//评论id数组
    var closed_status = 0;
    if(!ent_code || !id || !ids){
        return res.json(returnCode.BUSY);
    }
    //获取有效的评论id数组
    ForumComment.find({ent_code:ent_code,thread:id,_id:{
        $in:ids
    }}).select('_id').exec(function(err, docs) {
        if(err){
            return res.json(returnCode.BUSY);
        }
        if(!docs || docs.length == 0){
            return res.json(_.assign({data:true}, returnCode.SUCCESS));
        }
        var available_ids = [];
        _.forEach(docs,function(doc){
            available_ids.push(doc._id);
        });
        batchUpdateCommentStatus(ent_code,available_ids,closed_status,function(err,result){
            if(err){
                return res.json(returnCode.BUSY);
            }
            return res.json(_.assign({data:true}, returnCode.SUCCESS));
        });
    });
});

//批量删除帖子的评论
router.post('/threadManagement/threads/:id/comment/batchdel', function(req, res, next) {
    var ent_code = req.session.user.ent_code;
    var id = req.params.id;//帖子id
    var ids = req.body.ids;//评论id数组
    var delete_status = 2;
    if(!ent_code || !id || !ids){
        return res.json(returnCode.BUSY);
    }
    //获取有效的评论id数组
    ForumComment.find({ent_code:ent_code,thread:id,_id:{
        $in:ids
    }}).select('_id').exec(function(err, docs) {
        if(err){
            return res.json(returnCode.BUSY);
        }
        if(!docs || docs.length == 0){
            return res.json(_.assign({data:true}, returnCode.SUCCESS));
        }
        var available_ids = [];
        _.forEach(docs,function(doc){
            available_ids.push(doc._id);
        });
        batchUpdateCommentStatus(ent_code,available_ids,delete_status,function(err,result){
            if(err){
                return res.json(returnCode.BUSY);
            }
            return res.json(_.assign({data:true}, returnCode.SUCCESS));
        });
    });
});

//查询评论列表
router.get('/threadManagement/threads/:tid/comment/search', function(req, res, next) {
    var tid = req.params.tid || null,
        pageNo = req.query.pageNo || 1,
        pageSize = req.query.pageSize || 10,
        ent_code = req.session.user.ent_code,
        mid = req.query.mid,
        floor_start = req.query.floor_start,
        floor_end = req.query.floor_end,
        content = req.query.content || '';
    var asc = req.query.asc === 'true'?1:-1;
    var sort = {//默认排序
        floor:asc
    };
    if(req.query.sort){//指定排序字段
        sort = {};
        sort[req.query.sort] = asc;
        sort['_id'] = -1;
    }

    var conditions = {
        ent_code: ent_code,
        thread: tid,
        level:1,
        status:{
            $ne:2
        }
    };
    if (content) {
        conditions.content = {
            $regex: content,
            $options: 'i'
        };
    }
    if(floor_start && floor_end){
        conditions.floor = {
            $gte:floor_start,
            $lte:floor_end
        };
    }else if(floor_start){
        conditions.floor = {
            $gte:floor_start
        };
    }else if(floor_end){
        conditions.floor = {
            $lte:floor_end
        };
    }
    if (tid && mid) {
        forumCommentService.getCommentListByMidOrderBy(mid, conditions,sort, pageNo, pageSize, function(err, results) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                //判断是否已经点赞
                util.loadLevel(ent_code, results.items, function() {
                    res.json(_.assign(results, returnCode.SUCCESS));
                });
            }
        });
    } else if(tid){
        forumCommentService.getAllCommentOrderBy(conditions,sort, pageNo, pageSize, function(err, results) {
            if (err) {
                console.log(err);
                res.json(returnCode.BUSY);
            } else {
                //判断是否已经点赞
                util.loadLevel(ent_code, results.items, function() {
                    res.json(_.assign(results, returnCode.SUCCESS));
                });
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//删除帖子的评论(查询结果)
router.post('/threadManagement/threads/:tid/comment/search/deleteAll', function(req, res, next) {
    var tid = req.params.tid || null,
        mid = req.body.mid,
        floor_start = req.body.floor_start,
        floor_end = req.body.floor_end,
        content = req.body.content || '';
    var ent_code = req.session.user.ent_code;
    var status = 2;//删除状态为2
    var conditions = {
        ent_code: ent_code,
        thread: tid,
        level:1,
        status:{
            $ne:2
        }
    };
    if (content) {
        conditions.content = {
            $regex: content,
            $options: 'i'
        };
    }
    if(floor_start){
        conditions.floor = {
            $gte:floor_start
        };
    }
    if(floor_end){
        conditions.floor = {
            $lte:floor_end
        };
    }

    var callback = function(err, result) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(_.assign({
                data:result
            }, returnCode.SUCCESS));
        }
    };
    var deleteAll= function(condition,callback){
        ForumComment.find(conditions).select('_id').exec(function(err, docs) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                var ids=[];
                _.forEach(docs,function(doc){
                    ids.push(doc._id);
                });
                var chunks=_.chunk(ids, 100);
                var tasks = [];
                _.forEach(chunks,function(chunk){
                    tasks.push(function(cb){
                        batchUpdateCommentStatus(ent_code,chunk,status,function(err,result){
                            cb(err,result);
                        });
                    });
                });
                async.series(tasks,function(err){
                    callback(err, true);
                });
            }
        });
    };
    if (tid && mid) {
        forumUserService.searchMembersByMid(mid, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户
                var user_ids = [];
                for (var i in users) {
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    $in: user_ids
                }
                deleteAll(conditions,callback);
            }
        });
    } else if(tid){
        deleteAll(conditions,callback);
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//屏蔽帖子的评论(查询结果)
router.post('/threadManagement/threads/:tid/comment/search/closeAll', function(req, res, next) {
    var tid = req.params.tid || null,
        mid = req.body.mid,
        floor_start = req.body.floor_start,
        floor_end = req.body.floor_end,
        content = req.body.content || '';
    var ent_code = req.session.user.ent_code;
    var status = 0;//屏蔽的状态为0
    var conditions = {
        ent_code: ent_code,
        thread: tid,
        level:1,
        status:{
            $ne:2
        }
    };
    if (content) {
        conditions.content = {
            $regex: content,
            $options: 'i'
        };
    }
    if(floor_start){
        conditions.floor = {
            $gte:floor_start
        };
    }
    if(floor_end){
        conditions.floor = {
            $lte:floor_end
        };
    }

    var callback = function(err, result) {
        if (err) {
            console.error(err);
            res.json(returnCode.BUSY);
        } else {
            res.json(_.assign({
                data:result
            }, returnCode.SUCCESS));
        }
    };
    var closeAll= function(condition,callback){
        ForumComment.find(conditions).select('_id').exec(function(err, docs) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                var ids=[];
                _.forEach(docs,function(doc){
                    ids.push(doc._id);
                });
                var chunks=_.chunk(ids, 100);
                var tasks = [];
                _.forEach(chunks,function(chunk){
                    tasks.push(function(cb){
                        batchUpdateCommentStatus(ent_code,chunk,status,function(err,result){
                            cb(err,result);
                        });
                    });
                });
                async.series(tasks,function(err){
                    callback(err, true);
                });
            }
        });
    };
    if (tid && mid) {
        forumUserService.searchMembersByMid(mid, function(err, users) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                //查询对应用户
                var user_ids = [];
                for (var i in users) {
                    user_ids.push(users[i]._id);
                }
                conditions.from = {
                    $in: user_ids
                }
                closeAll(conditions,callback);
            }
        });
    } else if(tid){
        closeAll(conditions,callback);
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//查询帖子的收藏数
router.get('/threadManagement/threads/:tid/collects', function(req, res, next) {
    var tid = req.params.tid || null;
    var ent_code = req.session.user.ent_code;
    var conditions = {
        ent_code: ent_code,
        thread: tid
    };
    if (tid && ent_code) {
        ForumFollowThread.count(conditions, function(err, count) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(_.assign({data:(count||0)}, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//查询帖子的子文章数
router.get('/threadManagement/threads/:tid/subtotal', function(req, res, next) {
    var tid = req.params.tid || null;
    var ent_code = req.session.user.ent_code;
    var conditions = {
        ent_code: ent_code,
        pid: tid,
        status:1
    };
    if (tid && ent_code) {
        ForumThread.count(conditions, function(err, count) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(_.assign({data:(count||0)}, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//查询（话题/照片墙）的参与人数
router.get('/threadManagement/threads/:tid/joined/total', function(req, res, next) {
    var tid = req.params.tid || null;
    var ent_code = req.session.user.ent_code;
    if (tid && ent_code) {
        ForumThread.aggregate({
            $match: {
                ent_code: ent_code,
                pid: mongoose.Types.ObjectId(tid)
            }
        }, {
            $group: {
                _id: {
                    from: '$from'
                },
                count: {
                    $sum: 1
                }
            }
        }, function(err, data) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(_.assign({data:((data && data.length)||0)}, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});