'use strict';
//
var router = require('express').Router(),
    returnCode = require('../../utils/returnCode'),
    _ = require('lodash'),
    then = require('thenjs'),
    moment = require('moment'),
    mongoose = require('mongoose'),
    async = require('async');
//ģ    
var ForumThread = mongoose.model('ForumThread'),
    ForumComment = mongoose.model('ForumComment'),
    ForumPVLog = mongoose.model('ForumPVLog'),
    ForumFollowThread = mongoose.model('ForumFollowThread');
//    
var forumThreadService = require('../../service/forumThreadService'),
    forumUserService = require('../../service/forumUserService'),
    forumCommentService = require('../../service/forumCommentService'),
    forumAboutMEService = require('../../service/forumAboutMEService');

var thread_batch_prefix = 'forum_thread_management_batchoperate_';
var comment_batch_prefix = 'forum_comment_management_batchoperate_';

module.exports = function(app) {
    app.use('/admin/forum', router);
};

function getBatchStatus(key,callback){
    redis.get(key,function(err,store){
        if(err){
            return callback && callback(err);
        }
        //storeΪnullʾδ,-1ʾʧܣstatus=0ʾڽУstatus=1ʾ
        if(!store){
            return callback && callback(null,null);
        }
        try{
            var obj = JSON.parse(store);
            return callback && callback(null,obj);
        }catch(e){
            return callback && callback(e);
        }
    });
}
function setStatus(key,status,expire){
    var _status = status || {status:0};
    redis.set(key,JSON.stringify(_status),function(err){
        if(err){
            console.log(err);
        }
        redis.expire(key,expire || 180);
    });
}
function canStartIf(key,cannot,can){
    getBatchStatus(key,function(err,store){
        if(err){
            return cannot && cannot(err);
        }
        if(store && store.status === 0){//ڴ
            console.log('Ѿڴ');
            return cannot && cannot(null,true);
        }
        //ѽδʼ
        setStatus(key);
        return can && can();
    });
}

function getThreadBatchOperateConditions(req){
    var infoId = req.body.infoId;
    var tagId = req.body.tagId;
    var pid = req.body.pid;//id
    var content = req.body.content;//  
    var type = req.body.type;//1ͨ,,Ƭǽ
    var begin_time=req.body.begin_time;
    var end_time=req.body.end_time;
    var tab = req.body.tab;//ȫ,ٷ,Ƽ,ö,Ӿ,,
    var conditions = {
        ent_code: req.session.user.ent_code,
        level: 1,
        status: {
            $ne: 3
        }
    };
    if (infoId) {
        conditions.info = infoId;
    }
    if (tagId) {
        conditions.tag = {
            $in: [tagId]
        };
    }
    if (pid) {
        conditions.pid = pid;
        conditions.level = 2;
    }
    if (content) {
        conditions.$or = [
            {
                title:{
                    $regex: content,
                    $options: 'i'
                }
            },
            {
                content:{
                    $regex: content,
                    $options: 'i'
                }
            }
        ];
    }
    if (type) {
        conditions.type = Number(type);
    }
    if(tab){
        if(tab === 'admin'){//ٷ
            conditions.isPublishByBg = 1;
        }
        else if(tab === 'new_recommend'){//Ƽ
            conditions.new_recommend = 1;
        }
        else if(tab === 'top'){//ö
            conditions.top = 1;
        }
        else if(tab === 'recommend'){//Ӿ
            conditions.recommend = 1;
        }
        else if(tab === 'event'){//
            conditions.isEvent = 1;
        }
        else if(tab === 'closed'){//
            conditions.status = 0;
        }
    }
    if(begin_time && end_time){
        conditions.created = {$gte : begin_time, $lte : end_time};
    } else if(end_time){
        conditions.created = {$lte : end_time};
    } else if(begin_time){
        conditions.created = {$gte : begin_time};
    }
    return conditions;
}
function batchAsyncUpdateThread(req,res,handlFn){
    var conditions = getThreadBatchOperateConditions(req);
    var key = thread_batch_prefix + req.session.user.ent_code;
    var nickName = req.body.nickName;
    var mid = req.body.mid;
    var start_time = new Date().getTime();
    var pageSize = 100;
    var ingInterval = (function(){//ʱڽ
        var interval = null;
        return {
            start:function(){
                interval = setInterval(function(){
                    setStatus(key);
                },3000);
            },
            clear:function(){
                return interval && clearInterval(interval);
            }
        }
    })();
    var callback = function(err) {
        ingInterval.clear();
        if (err) {//ʧ
            console.log(err);
            setStatus(key,{status:-1},1800);
        } else {//
            setStatus(key,{status:1,start:start_time,end:new Date().getTime()},1800);
        }
    };
    var progressAll= function(condition,callback){
        ForumThread.find(conditions).count(function(err,count){
            if(err || !count){
                return callback(err, true);
            }
            var tasks = [];
            var page_total = Math.ceil((count || 0) / pageSize) + 1;
            _.each(_.range(page_total),function(pageNo){
                tasks.push(function(cb){
                    ForumThread.find(conditions).select('_id').skip((pageNo)*pageSize).limit(pageSize)
                        .exec(function(err, docs) {
                            if (err) {
                                return cb(err, null);
                            }
                            var chunks=[];
                            if(docs && docs.length>0){
                                _.forEach(docs,function(doc){
                                    chunks.push(function(cb2){
                                        handlFn(doc,cb2);
                                    });
                                });
                            }
                            async.parallel(chunks,function(err){
                                cb(err, true);
                            });
                        });
                });
            });
            async.series(tasks,function(err){
                callback(err, true);
            });
        });
        ingInterval.start();
    };
    canStartIf(key,function(err,handling){
        if(err){
            console.log(err);
            return res.json(returnCode.BUSY);
        }
        if(handling){//ڴ
            return res.json(_.assign({
                data:false
            }, returnCode.SUCCESS));
        }
    },function(){
        if (mid) {
            forumUserService.searchMembersByMid(mid, function(err, users) {
                if (err) {
                    callback(err, null);
                } else {
                    //ѯӦû
                    var user_ids = [];
                    for(var i in users){
                        user_ids.push(users[i]._id);
                    }
                    conditions.from = {
                        "$in" : user_ids
                    };
                    progressAll(conditions,callback);
                }
            });
        } else if (nickName) {
            //ѯû
            forumUserService.searchMembersByNickName(nickName, function(err, users) {
                if (err) {
                    callback(err, null);
                } else {
                    //ѯӦû
                    var user_ids = [];
                    for(var i in users){
                        user_ids.push(users[i]._id);
                    }
                    conditions.from = {
                        "$in" : user_ids
                    };
                    progressAll(conditions,callback);
                }
            });
        } else {
            progressAll(conditions,callback);
        }
        res.json(_.assign({data:true}, returnCode.SUCCESS));
    });
}
router.get('/threadManagement/threads/list/batch/result',function(req,res,next){
    var key = thread_batch_prefix + req.session.user.ent_code;
    getBatchStatus(key,function(err,store){
        if(err){
            return res.json(returnCode.BUSY);
        }
        res.json(_.assign({
            data:store
        }, returnCode.SUCCESS));
    });
});
//ɾѯ
router.post('/threadManagement/threads/list/deleteAllAsync', function(req, res, next) {
    batchAsyncUpdateThread(req,res,function(thread,cb){
        forumThreadService.logicDeleteThreadById(thread._id, function(err) {
            if (err) {
                cb(err, null);
            } else {
                forumAboutMEService.updateThreadStatus(doc._id, 3);
                cb(null, true);
            }
        });
    })
});
//βѯ
router.post('/threadManagement/threads/list/closeAllAsync', function(req, res, next) {
    batchAsyncUpdateThread(req,res,function(thread,cb){
        forumThreadService.updateThreadById(thread._id, {status:0}, function(err) {
            if (err) {
                cb(err, null);
            } else {
                forumAboutMEService.updateThreadStatus(thread._id, 0);
                cb(null, true);
            }
        });
    })
});

function updateCommentStatus(ent_code,comment,status,cb){
    if(!ent_code || !comment){
        return cb && cb(null,true);
    }
    var cid = comment._id;
    var tid = comment.thread;
    async.parallel([
        function(callback) {
            //״̬
            forumCommentService.updateCommentStatusById(cid, status, function(err) {
                callback(err);
            });
        },
        function(callback){
            forumAboutMEService.updateCommentLevel1Status(cid, status);
            callback();
        },
        function(callback) {
            //Ƿɾ
            if (status == 2 && tid) {
                forumThreadService.updateThreadById(tid, {
                    $pull:{
                        comments:cid
                    },
                    $inc: {
                        comment_count: -1
                    }
                }, function(err) {
                    callback(err);
                });
            } else {
                callback();
            }
        }
    ], function(err) {
        cb(err);
    });
}
function batchAsyncUpdateCommentStatus(status,req,res){
    var conditions = getCommentBatchOperateConditions(req);
    var ent_code = req.session.user.ent_code;
    var tid = req.params.tid;
    var key = comment_batch_prefix + ent_code + tid;
    var mid = req.body.mid;
    var status = status;//status=0Σstatus=2ɾ
    var start_time = new Date().getTime();
    var pageSize = 100;
    var ingInterval = (function(){//ʱڽ
        var interval = null;
        return {
            start:function(){
                interval = setInterval(function(){
                    setStatus(key);
                },3000);
            },
            clear:function(){
                return interval && clearInterval(interval);
            }
        }
    })();
    var callback = function(err) {
        ingInterval.clear();
        if (err) {//ʧ
            console.log(err);
            setStatus(key,{status:-1},1800);
        } else {//
            setStatus(key,{status:1,start:start_time,end:new Date().getTime()},1800);
        }
    };
    var progressAll= function(condition,callback){
        ForumComment.find(conditions).count(function(err,count){
            if(err || !count){
                return callback(err, true);
            }
            var tasks = [];
            var page_total = Math.ceil((count || 0) / pageSize) + 1;
            _.each(_.range(page_total),function(pageNo){
                tasks.push(function(cb){
                    ForumComment.find(conditions).select('_id thread').skip((pageNo)*pageSize).limit(pageSize)
                        .exec(function(err, docs) {
                            if (err) {
                                return cb(err, null);
                            }
                            var chunks=[];
                            if(docs && docs.length>0){
                                _.forEach(docs,function(doc){
                                    chunks.push(function(cb2){
                                        updateCommentStatus(ent_code,doc,status,function(err){
                                            cb2(err,null);
                                        })
                                    });
                                });
                            }
                            async.parallel(chunks,function(err){
                                cb(err, true);
                            });
                        });
                });
            });
            async.series(tasks,function(err){
                callback(err, true);
            });
        });
        ingInterval.start();
    };
    canStartIf(key,function(err,handling){
        if(err){
            console.log(err);
            return res.json(returnCode.BUSY);
        }
        if(handling){//ڴ
            return res.json(_.assign({
                data:false
            }, returnCode.SUCCESS));
        }
    },function(){
        if (mid) {
            forumUserService.searchMembersByMid(mid, function(err, users) {
                if (err) {
                    callback(err, null);
                } else {
                    //ѯӦû
                    var user_ids = [];
                    for (var i in users) {
                        user_ids.push(users[i]._id);
                    }
                    conditions.from = {
                        $in: user_ids
                    }
                    progressAll(conditions,callback);
                }
            });
        } else {
            progressAll(conditions,callback);
        }
        res.json(_.assign({data:true}, returnCode.SUCCESS));
    });
}
function getCommentBatchOperateConditions(req){
    var tid = req.params.tid || null,
        floor_start = req.body.floor_start,
        floor_end = req.body.floor_end,
        content = req.body.content || '';
    var ent_code = req.session.user.ent_code;
    var conditions = {
        ent_code: ent_code,
        thread: tid,
        level:1,
        status:{
            $ne:2
        }
    };
    if (content) {
        conditions.content = {
            $regex: content,
            $options: 'i'
        };
    }
    if(floor_start){
        conditions.floor = {
            $gte:floor_start
        };
    }
    if(floor_end){
        conditions.floor = {
            $lte:floor_end
        };
    }
    return conditions;
}
router.get('/threadManagement/threads/:tid/comment/list/batch/result',function(req,res,next){
    var key = comment_batch_prefix + req.session.user.ent_code + req.params.tid;
    getBatchStatus(key,function(err,store){
        if(err){
            return res.json(returnCode.BUSY);
        }
        res.json(_.assign({
            data:store
        }, returnCode.SUCCESS));
    });
});
//첽ɾӵ(ѯ)
router.post('/threadManagement/threads/:tid/comment/search/deleteAllAsync', function(req, res, next) {
    batchAsyncUpdateCommentStatus(2,req,res);
});
//첽ӵ(ѯ)
router.post('/threadManagement/threads/:tid/comment/search/closeAllAsync', function(req, res, next) {
    batchAsyncUpdateCommentStatus(0,req,res);
});